use shared::blog_post;
use worker::*;
use serde::Deserialize;

#[allow(dead_code)]
#[derive(Deserialize, Debug)]
struct R2Object {
    key: String,
    size: u64,
    #[serde(rename = "eTag")]
    e_tag: String,
}

#[allow(dead_code)]
#[derive(Debug, Deserialize)]
struct R2Message {
    account: String,
    action: String,
    bucket: String,
    object: R2Object,
}

#[event(queue)]
async fn queue(message_batch: MessageBatch<R2Message>, env: Env, _ctx: Context) -> Result<()> {
    let index = env.d1("INDEX")?;
    let bucket = env.bucket("GABRIELSIMMERCOM_BUCKET")?;
    let messages = message_batch.messages()?;
    for message in messages {
        let fname = message.into_body().object.key;
        let dir: Vec<&str> = fname.split("/").collect();
        if dir[0] != "posts" {
            continue
        }
        console_log!("got {} to render and index", &fname);
        let file = bucket.get(&fname).execute().await?;
        if file.is_none() {
            console_warn!("did not find a file in bucket for {}", fname)
        }
        let rendered = blog_post(fname.clone(), file.unwrap().body().unwrap().text().await?).unwrap();

        let statement = index.prepare("INSERT INTO posts (slug, title, html, date) VALUES (?1, ?2, ?3, ?4) ON CONFLICT(slug) DO UPDATE SET html=?3;");
        let query = statement.bind(&[
            rendered.slug.into(), rendered.title.into(), rendered.html.into(), rendered.date.into()
        ]).unwrap();
        let _result = query.run().await?;
    }

    Ok(())
}
